package com.tencent.qqlivetv.accountcenter;

import java.util.ArrayList;

import android.app.Service;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.IBinder;
import android.os.RemoteException;
import android.text.TextUtils;
import android.util.Log;

public class AccountCenterProxy {
    private static final String TAG = "AccountCenterProxy";
    
    public interface IAccountConnectionListener {
        public void onConnectionStatusChanged(boolean connected); 
    }
    
    private static AccountCenterProxy s_Instance = null;
    
    private IAccountCenter mAccountService = null;
    private boolean mConnected = false;
    private boolean mBindServiceSucceed = false;
    private Context mContext = null;
    private IAccountConnectionListener mAccountConnectionListener = null;
    
    private ServiceConnection mConnection = new ServiceConnection() {
        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            Log.i(TAG, "onServiceConnected");
            mAccountService = IAccountCenter.Stub.asInterface(service);
            mConnected = true;
            if (mAccountConnectionListener != null) {
                mAccountConnectionListener.onConnectionStatusChanged(true);
            }
        }

        @Override
        public void onServiceDisconnected(ComponentName name) {
            Log.i(TAG, "onServiceDisconnected");
            mAccountService = null;
            mConnected = false;
            if (mAccountConnectionListener != null) {
                mAccountConnectionListener.onConnectionStatusChanged(false);
            }
            connectService();
        }
    };
    
    public static AccountCenterProxy getInstance(Context context) {
        if (s_Instance == null && context != null) {
            synchronized (AccountCenterProxy.class) {
                if (s_Instance == null) {
                    s_Instance = new AccountCenterProxy(context);
                }
            }
        }
        
        return s_Instance;
    }
    
    public AccountCenterProxy(Context context) {
        mContext = context;
    }
    
    public void init(IAccountConnectionListener listener) {
        mAccountConnectionListener = listener;
        connectService();
    }
    
    private void connectService() {
        if (mContext == null) {
            return;
        }
        Log.i(TAG, "connectService");
        
        Intent intent = new Intent();
        intent.setAction("com.tencent.qqlivetv.action.ACCOUNT_SERVICE");
        try {
            mBindServiceSucceed = mContext.getApplicationContext().bindService(intent, mConnection, Service.BIND_AUTO_CREATE);
            if (!mBindServiceSucceed) {
                Log.w(TAG, "bindService failed");
            }
        } catch (SecurityException e) {
        	mBindServiceSucceed = false;
            Log.e(TAG, "bindService failed, SecurityException: " + e);
        }
    }
    
    public void setExpired(String appName, String openId, String md5) {
        if (TextUtils.isEmpty(appName)) {
            Log.e(TAG, "setExpired: appName is empty!");
            return;
        }
        
        if (TextUtils.isEmpty(openId)) {
            Log.e(TAG, "setExpired: openId is empty!");
            return;
        }
        
        if (TextUtils.isEmpty(md5)) {
            Log.e(TAG, "setExpired: md5 is empty!");
            return;
        }
        
        if (!mConnected || mAccountService == null) {
            Log.e(TAG, "setExpired: NOT connected!");
            return;
        }
        
        Log.i(TAG, "setExpired: " + appName + ", " + openId + ", " + md5);
        
        try {
            mAccountService.setExpired(appName, openId, md5);
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }
    
    public void logout(String appName) {
        if (TextUtils.isEmpty(appName)) {
            Log.e(TAG, "logout: appName is empty!");
            return;
        }
        
        if (!mConnected || mAccountService == null) {
            Log.e(TAG, "logout: NOT connected!");
            return;
        }
        
        Log.i(TAG, "logout: " + appName);
        
        try {
            mAccountService.logout(appName);
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }
    
    public AccountInfo getAccountInfo(String appName) {
        AccountInfo info = null;
        if (TextUtils.isEmpty(appName)) {
            Log.e(TAG, "getAccountInfo: appName is empty!");
            return info;
        }
        
        if (!mConnected || mAccountService == null) {
            Log.e(TAG, "getAccountInfo: NOT connected!");
            return info;
        }
        
        Log.i(TAG, "getAccountInfo: " + appName);
        
        try {
            info = mAccountService.getAccountInfo(appName);
        } catch (RemoteException e) {
            e.printStackTrace();
        }
        
        return info;
    }
    
    public ArrayList<AccountInfo> getAccountList(int count) {
        if (count <= 0) {
            Log.e(TAG, "getAccountList: count invalid: " + count);
            return null;
        }
        
        if (!mConnected || mAccountService == null) {
            Log.e(TAG, "getAccountList: NOT connected!");
            return null;
        }
        
        String accountListJsonString = "";
        try {
            accountListJsonString = mAccountService.getAccountList(count);
        } catch (RemoteException e) {
            e.printStackTrace();
        }
        
        return AccountUtils.jsonToList(accountListJsonString);
    }
    
    public boolean isConnected() {
        return mConnected;
    }
    
    public void destory() {
        Log.i(TAG, "destory");
        try {
            if (mContext != null && mBindServiceSucceed) {
                mContext.getApplicationContext().unbindService(mConnection);
                mBindServiceSucceed = false;
            }
		} catch (Exception e) {
			Log.i(TAG, e.getMessage());
		}

    }
}